/*
 *	recite - english text speech synthesizer
 *	Copyright (C) 1993 Peter Miller.
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 1, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * MANIFEST: functions to use workstation's speaker
 *
 * Portions of this file are derived from a program
 * by  H.F. Silverman (Jan-91) and A. Smith (Feb-91),
 * no copyright notice present.
 *
 * Modified for Linux/OSS Tue 15 Dec 1998
 * by Charles Briscoe-Smith <cpbs@debian.org>
 *
 * Modified for OSS buffering issues Thu 27 Mar 2003
 * by Sam Hocevar <sam@zoy.org>
 */

#include <stdio.h>

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/select.h>

#include "audio.h"
#include "error.h"
#include "mem.h"


static double volume;


void
ulaw_volume(n)
	double	n;
{
	volume = n;
}


void
ulaw_read(filename, data_p, datalen_p)
	char		*filename;
	char		**data_p;
	long		*datalen_p;
{
	int		fd;
	char		*data;
	size_t		data_max;
	size_t		datalen;

	/*
	 * open the file
	 */
	if (filename)
	{
		fd = open(filename, O_RDONLY, 0666);
		if (fd < 0)
			nfatal("could not open \"%s\"", filename);
	}
	else
	{
		fd = 0;
		filename = "(standard input)";
	}

	/*
	 * read the data
	 */
	data_max = (1L << 18);
	data = mem_alloc(data_max);
	datalen = 0;
	for (;;)
	{
		long	rsize;
		long	nbytes;

		if (datalen >= data_max)
		{
			rsize = (1L << 18);
			data_max += rsize;
			mem_change_size(&data, data_max);
		}
		else
			rsize = data_max - datalen;
		nbytes = read(fd, data + datalen, rsize);
		if (nbytes < 0)
			nfatal("could not read from \"%s\"", filename);
		if (!nbytes)
			break;
		datalen += nbytes;
	}
	if (datalen < data_max)
		mem_change_size(&data, datalen);
	*data_p = data;
	*datalen_p = datalen;

	/*
	 * close the file
	 */
	if (close(fd))
		nfatal("could not close \"%s\"", filename);
}


void
ulaw_write(filename, data, datalen)
	char		*filename;
	char		*data;
	long		datalen;
{
	int		fd;

	if (filename)
	{
		fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0666);
		if (fd < 0)
			nfatal("could not create \"%s\"", filename);
	}
	else
	{
		fd = 1;
		filename = "(standard output)";
	}

	if (write(fd, data, datalen) < 0)
		nfatal("could not write to \"%s\"", filename);

	if (close(fd))
		nfatal("could not close \"%s\"", filename);
}


void
ulaw_play(jack, data, datalen)
	int	jack;
	char	*data;
	long	datalen;
{
        int             err;
	char            *Audio_dev = "/dev/dsp";
	int             fd;	/* file descriptor for audio device */
	audio_buf_info  info;   /* audio device buffer info */
	int             empty_bytes; /* available bytes in the device */

	/*
	 * Try it quickly, first
	 */
        fd = open(Audio_dev, O_WRONLY | O_NDELAY, 0666);
#if 1
        if (fd < 0 && errno == EBUSY)
	{
		error("waiting for %s", Audio_dev);
		fd = open(Audio_dev, O_WRONLY, 0666);
	}
#endif
	if (fd < 0)
		nfatal("could not open \"%s\"", Audio_dev);

	/* Remember how many available bytes we have when the buffer is empty */
	if(ioctl(fd, SNDCTL_DSP_GETOSPACE, &info) == 0)
		empty_bytes = info.bytes;
	else
		empty_bytes = -1;

	while (datalen > 0)
	{
		fd_set fdset;
		struct timeval timeout;

		/* Wait for device to become available */
		FD_ZERO(&fdset);
		FD_SET(fd, &fdset);
		timeout.tv_sec = 2;
		timeout.tv_usec = 0;
		if(select(fd+1, NULL, &fdset, NULL, &timeout) <= 0)
			nfatal("audio device \"%s\" is unavailable, check "
			       "that no other program is using it", Audio_dev);

		/* Write to device */
		err = write(fd, data, datalen);
		if (err == -1)
		{
			if(errno != EAGAIN)
				nfatal("could not write to \"%s\"", Audio_dev);

			/* Duh, looks like it wasn't really available */
			usleep(10000);
			err = 0;
		}
		datalen -= err;
		data += err;
	}

	/* Wait for the samples to be played before closing the device. */
	for(;;)
	{
		if(empty_bytes >= 0 &&
		    ioctl(fd, SNDCTL_DSP_GETOSPACE, &info) == 0 &&
		    info.bytes >= empty_bytes)
			break;
		usleep(10000);
	}

        close(fd);
}
